<?php
/*
Plugin Name: User Locker
Plugin URI: http://www.poradnik-webmastera.com/projekty/user_locker/
Description: This plugin locks user account after given number of incorrect login attempts.
Author: Daniel Frużyński
Version: 1.0
Author URI: http://www.poradnik-webmastera.com/
Text Domain: user-locker
*/

/*  Copyright 2009  Daniel Frużyński  (email : daniel [A-T] poradnik-webmastera.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


if ( !class_exists( 'UserLocker' ) ) {
	class UserLocker {
		var $pass_check = false;
		var $locked = false;
		
		// Constructor
		function UserLocker() {
			// Initialise plugin
			add_action( 'init', array( &$this, 'init' ) );
			add_action( 'admin_init', array( &$this, 'admin_init' ) );
			
			// Add option to Admin menu
			add_action( 'admin_menu', array( &$this, 'admin_menu' ) );
			
			// Check if user is already locked
			add_filter( 'wp_authenticate_user', array( &$this, 'wp_authenticate_user' ), 1 );
			
			// Set password check flag
			add_filter( 'check_password', array( &$this, 'check_password' ) );

			// Increment bad attempt counter and finally lock account
			add_action( 'wp_login_failed', array( &$this, 'wp_login_failed' ) );
			
			// Reset account lock on pass reset and valid login
			add_action( 'password_reset', array( &$this, 'password_reset' ) );
			add_action( 'wp_login', array( &$this, 'wp_login' ) );
			
			// Add info about account lock
			add_filter( 'login_errors', array( &$this, 'login_errors' ) );
		}
		
		// Initialise plugin
		function init() {
			if ( function_exists( 'load_plugin_textdomain' ) ) {
				load_plugin_textdomain( 'user-locker', PLUGINDIR.'/'.dirname(plugin_basename(__FILE__)) );
			}
		}
		
		// Initialise plugin - admin part
		function admin_init() {
			register_setting( 'user-locker', 'userlocker_max_attempts', array( &$this, 'sanitize_nonnegative' ) );
		}
		
		// Add option to Admin menu
		function admin_menu() {
			$file = __FILE__;
			
			// hack for 1.5
			global $wp_version;
			if ( '1.5' == substr( $wp_version, 0, 3 ) ) {
				$file = 'user-locker/user-locker.php';
			}
			add_submenu_page( 'options-general.php', 'User Locker', 
				'User Locker', 10, $file, array( $this, 'options_panel' ) );
		}
		
		// Check if user is already locked
		function wp_authenticate_user( $user ) {
			if ( is_wp_error( $user ) ) {
				return $user;
			}
			
			$locked = get_user_option( 'ul_locked', $user->ID, false );
			if ( $locked ) {
				return new WP_Error('ul_user_locked', __('<strong>ERROR</strong>: This user account is locked for security reasons. Please use Lost Password option to unlock it.', 'user-locker'));
			}
			
			return $user;
		}
		
		// Set password check flag
		function check_password( $check ) {
			$this->pass_check = true;
			return $check;
		}
		
		// Increment bad attempt counter and finally lock account
		function wp_login_failed( $username ) {
			if ( !$this->pass_check ) {
				// Function called too early
				return;
			}
			
			$user = get_userdatabylogin($username);
			if ( !$user || ($user->user_login != $username) ) {
				// Invalid username
				return;
			}
			
			$locked = get_user_option( 'ul_locked', $user->ID, false );
			if ( !$locked ) {
				$cnt = get_user_option( 'ul_bad_attempts', $user->ID, false );
				if ( $cnt === false ) {
					$cnt = 1;
				} else {
					++$cnt;
				}
				update_user_option( $user->ID, 'ul_bad_attempts', $cnt, false );
				
				if ( $cnt >= get_option( 'userlocker_max_attempts' ) ) {
					$this->locked = true;
					update_user_option( $user->ID, 'ul_locked', true, false );
				}
			}
		}
		
		// Reset account lock on pass reset
		function password_reset( $user ) {
			$this->unlock_user( $user->ID );
		}
		
		// Reset account lock on valid login
		function wp_login( $username ) {
			$user = get_userdatabylogin( $username );
			$this->unlock_user( $user->ID );
		}
		
		// Unlock account for given user
		function unlock_user( $user_id ) {
			update_user_option( $user_id, 'ul_bad_attempts', 0, false );
			update_user_option( $user_id, 'ul_locked', false, false );
		}
		
		// Add info about account lock
		function login_errors( $errors ) {
			if ( $this->locked ) {
				$errors .= __('<strong>ERROR</strong>: This user account has been locked for security reasons. Please use Lost Password option to unlock it.', 'user-locker') . "<br />\n";
			}
			return $errors;
		}
		
		function sanitize_nonnegative( $value ) {
			$value = (int)$value;
			if ( $value < 0 ) {
				$value = 0;
			}
			return $value;
		}
		
		// Display settings form
		function options_panel() {
			$message = null;
			if ( isset($_POST['action']) ) {
				check_admin_referer( 'user-locker-options' );
				$message = __('Configuration has been saved.', 'user-locker');
				echo '<div id="message" class="updated fade"><p>', $message, '</p></div>', "\n";
			}
?>
<div id="dropmessage" class="updated" style="display:none;"></div>
<div class="wrap">
<h2><?php _e('User Locker - Options', 'user-locker'); ?></h2>

<form name="dofollow" action="options.php" method="post">
<?php settings_fields( 'user-locker' ); ?>
<table class="form-table">

<tr><th colspan="2"><h3><?php _e('Account locking:', 'user-locker'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="userlocker_max_attempts"><?php _e('Maximum invalid login attempts before account locking:', 'user-locker'); ?></label>
</th>
<td>
<input type="text" maxlength="6" size="10" id="userlocker_max_attempts" name="userlocker_max_attempts" value="<?php echo stripcslashes( get_option( 'userlocker_max_attempts' ) ); ?>" />
</td>
</tr>

</table>

<p class="submit">
<input type="hidden" name="action" value="update" />
<input type="submit" name="Submit" value="<?php _e('Save settings', 'user-locker'); ?>" /> 
</p>

</form>
</div>
<?php
		}
	}
	
	add_option( 'userlocker_max_attempts', 5 ); // Maximum invalid login attempts before account locking
	
	$wp_user_locker = new UserLocker();
}

?>